
import urllib
import urllib2
import datetime

from django.conf import settings
from django.template.loader import render_to_string

from billing.models import AuthorizeAIMResponse
from billing import Gateway
from billing.signals import *
from billing.utils.credit_card import InvalidCard, Visa, \
    MasterCard, Discover, AmericanExpress
from billing.utils.xml_parser import parseString, nodeToDic

API_VERSION = '3.1'
DELIM_CHAR = ','
ENCAP_CHAR = '$'
APPROVED, DECLINED, ERROR, FRAUD_REVIEW = 1, 2, 3, 4
RESPONSE_CODE, RESPONSE_REASON_CODE, RESPONSE_REASON_TEXT = 0, 2, 3


def save_authorize_response(response):
    data = {}
    data['response_code']                 = int(response[0])
    data['response_reason_code']          = response[2]
    data['response_reason_text']          = response[3]
    data['authorization_code']            = response[4]
    data['address_verification_response'] = response[5]
    data['transaction_id']                = response[6]
    data['invoice_number']                = response[7]
    data['description']                   = response[8]
    data['amount']                        = response[9]
    data['method']                        = response[10]
    data['transaction_type']              = response[11]
    data['customer_id']                   = response[12]

    data['first_name'] = response[13]
    data['last_name']  = response[14]
    data['company']    = response[15]
    data['address']    = response[16]
    data['city']       = response[17]
    data['state']      = response[18]
    data['zip_code']   = response[19]
    data['country']    = response[20]
    data['phone']      = response[21]
    data['fax']        = response[22]
    data['email']      = response[23]


    data['shipping_first_name'] = response[24]
    data['shipping_last_name']  = response[25]
    data['shipping_company']    = response[26]
    data['shipping_address']    = response[27]
    data['shipping_city']       = response[28]
    data['shipping_state']      = response[29]
    data['shipping_zip_code']   = response[30]
    data['shipping_country']    = response[31]
 
    data['card_code_response'] = response[38]
    return AuthorizeAIMResponse.objects.create(**data)
    

class AuthorizeNetGateway(Gateway):
    test_url = "https://test.authorize.net/gateway/transact.dll"
    live_url = "https://secure.authorize.net/gateway/transact.dll"

    arb_test_url = 'https://apitest.authorize.net/xml/v1/request.api'
    arb_live_url = 'https://api.authorize.net/xml/v1/request.api'

    supported_countries = ["US"]
    default_currency = "USD"

    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    homepage_url = "http://www.authorize.net/"
    display_name = "Authorize.Net"

    def __init__(self):
        self.login = settings.AUTHORIZE_LOGIN_ID
        self.password = settings.AUTHORIZE_TRANSACTION_KEY
    
    def add_invoice(self, post, options):
        """add invoice details to the request parameters"""
        post['invoice_num'] = options.get('order_id', None)
        post['description'] = options.get('description', None)
    
    def add_creditcard(self, post, credit_card):
        """add credit card details to the request parameters"""
        post['card_num']   = credit_card.number
        post['card_code']  = credit_card.verification_value
        post['exp_date']   = credit_card.expire_date
        post['first_name'] = credit_card.first_name
        post['last_name']  = credit_card.last_name
    
    def add_address(self, post, options):
        """add billing/shipping address details to the request parameters"""
        if options.get('billing_address', None):
            address = options.get('billing_address')
            post['address']  = address.get('address1', '') + \
                               address.get('address2', '')
            post['company']  = address.get('company', '')
            post['phone']    = address.get('phone', '')
            post['zip']      = address.get('zip', '')
            post['city']     = address.get('city', '')
            post['country']  = address.get('country', '')
            post['state']   = address.get('state', '')
          
        if options.get('shipping_address', None):
            address = options.get('shipping_address')
            post['ship_to_first_name'] = address.get('name', '').split(" ")[0]
            post['ship_to_last_name']  = " ".join(address.get('name', '').split(" ")[1:])
            post['ship_to_address']    = address.get('address1', '') + \
                                         address.get('address2', '')
            post['ship_to_company']    = address.get('company', '')
            post['ship_to_phone']      = address.get('phone', '')
            post['ship_to_zip']        = address.get('zip', '')
            post['ship_to_city']       = address.get('city', '')
            post['ship_to_country']    = address.get('country', '')
            post['ship_to_state']      = address.get('state', '')
    
    def add_customer_data(self, post, options):
        """add customer details to the request parameters"""
        if options.has_key('email'):
            post['email'] = options['email']
            post['email_customer'] = True
        
        if options.has_key('customer'):
            post['cust_id'] = options['customer']

        if options.has_key('ip'):
            post['customer_ip'] = options['ip']

    @property
    def service_url(self):
        if self.test_mode:
            return self.test_url
        return self.live_url

    def commit(self, action, money, parameters):
        if not action == 'VOID':
            parameters['amount'] = money
        
        parameters['test_request'] =  self.test_mode
        url = self.service_url
        data = self.post_data(action, parameters)
        response = self.request(url, data)
        return response

    def post_data(self, action, parameters = None):
        """add API details, gateway response formating options 
        to the request parameters"""
        if not parameters:
            parameters = {}
        post = {}
        
        post['version']        = API_VERSION
        post['login']          = self.login
        post['tran_key']       = self.password
        post['relay_response'] = "FALSE"
        post['type']           = action
        post['delim_data']     = "TRUE"
        post['delim_char']     = DELIM_CHAR
        post['encap_char']     = ENCAP_CHAR
        
        post.update(parameters)
        return urllib.urlencode(dict(('x_%s' % (k), v) for k, v in post.iteritems()))
    
    # this shoud be moved to a requests lib file
    def request(self, url, data, headers = None):
        """Make POST request to the payment gateway with the data and return 
        gateway RESPONSE_CODE, RESPONSE_REASON_CODE, RESPONSE_REASON_TEXT"""
        if not headers:
            headers = {}
        conn = urllib2.Request(url=url, data=data, headers=headers)
        try:
            open_conn = urllib2.urlopen(conn)
            response = open_conn.read()
        except urllib2.URLError:
            return (5, '1', 'Could not talk to payment gateway.')
        fields = response[1:-1].split('%s%s%s' % (ENCAP_CHAR, DELIM_CHAR, ENCAP_CHAR))
        return save_authorize_response(fields)

    def purchase(self, money, credit_card, options = None):
        """Using Authorize.net payment gateway, charge the given
        credit card for specified money"""
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")

        post = {}     
        self.add_invoice(post, options) 
        self.add_creditcard(post, credit_card) 
        self.add_address(post, options)
        self.add_customer_data(post, options)

        response = self.commit("AUTH_CAPTURE", money, post)
        status = "SUCCESS"
        if response.response_code != 1:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self, 
                                              type="purchase",
                                              response="response")
        else:
            transaction_was_successful.send(sender=self,
                                            type="purchase",
                                            response=response)
        return {"status": status, "response": response}
    
    def authorize(self, money, credit_card, options = None):
        """Using Authorize.net payment gateway, authorize the
        credit card for specified money"""
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")

        post = {}
        self.add_invoice(post, options) 
        self.add_creditcard(post, credit_card) 
        self.add_address(post, options)
        self.add_customer_data(post, options)

        response = self.commit("AUTH_ONLY", money, post)
        status = "SUCCESS"
        if response.response_code != 1:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self, 
                                              type="authorization",
                                              response=response)
        else:
            transaction_was_successful.send(sender=self,
                                            type="authorization",
                                            response=response)
        return {"status": status, "response": response}
    
    def capture(self, money, authorization, options = None):
        """Using Authorize.net payment gateway, capture the
        authorize credit card"""
        if not options:
            options = {}
        post = {}        
        post["trans_id"] = authorization
        post.update(options)

        response = self.commit("PRIOR_AUTH_CAPTURE", money, post)
        status = "SUCCESS"
        if response.response_code != 1:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="capture",
                                              response=response)
        else:
            transaction_was_successful.send(sender=self,
                                            type="capture",
                                            response=response)
        return {"status": status, "response": response}
    
    def void(self, identification, options = None):
        """Using Authorize.net payment gateway, void the
        specified transaction"""
        if not options:
            options = {}
        post = {}        
        post["trans_id"] = identification
        post.update(options)

        response = self.commit("VOID", money, post)
        status = "SUCCESS"
        if response.response_code != 1:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="void",
                                              response=response)
        else:
            transaction_was_successful.send(sender=self,
                                            type="void",
                                            response=response)
        return {"status": status, "response": response}

    def credit(self, money, identification, options = None):
        """Using Authorize.net payment gateway, void the
        specified transaction"""
        if not options:
            options = {}
        post = {}
        post["trans_id"] = identification
        # Authorize.Net requuires the card or the last 4 digits be sent
        post["card_num"] = options["credit_card"]
        post.update(options)

        response = self.commit("CREDIT", money, post)
        status = "SUCCESS"
        if response.response_code != 1:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="credit",
                                              response=response)
        else:
            transaction_was_successful.send(sender=self,
                                            type="credit",
                                            response=response)
        return {"status": status, "response": response}

    def recurring(self, money, credit_card, options):
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")
        template_vars  = {}
        template_vars['auth_login']  = self.login
        template_vars['auth_key']    = self.password
        template_vars['amount']      = money
        template_vars['card_number'] = credit_card.number
        template_vars['exp_date']    = credit_card.expire_date
        
        template_vars['start_date']        = options.get('start_date') or datetime.date.today().strftime("%Y-%m-%d")
        template_vars['total_occurrences'] = options.get('total_occurences', 9999)
        template_vars['interval_length']   = options.get('interval_length', 1)
        template_vars['interval_unit']     = options.get('interval_unit', 'months')
        template_vars['sub_name']          = options.get('sub_name', '')
        template_vars['first_name']        = credit_card.first_name
        template_vars['last_name']         = credit_card.last_name
        
        xml = render_to_string('billing/arb/arb_create_subscription.xml', template_vars)
        
        if self.test_mode:
            url = self.arb_test_url
        else:
            url = self.arb_live_url
        headers = {'content-type':'text/xml'}
        
        conn = urllib2.Request(url=url, data=xml, headers=headers)
        try:
            open_conn = urllib2.urlopen(conn)
            xml_response = open_conn.read()
        except urllib2.URLError:
            return (5, '1', 'Could not talk to payment gateway.')
        
        response = nodeToDic(parseString(xml_response))['ARBCreateSubscriptionResponse']
        # successful response
        # {u'ARBCreateSubscriptionResponse': {u'messages': {u'message': {u'code': u'I00001', 
        #                                                               u'text': u'Successful.'},
        #                                                  u'resultCode': u'Ok'},
        #                                    u'subscriptionId': u'933728'}}
        
        status = "SUCCESS"
        if response['messages']['resultCode'].lower() != 'ok':
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="recurring",
                                              response=response)
        else:
            transaction_was_successful.send(sender=self,
                                            type="recurring",
                                            response=response)
        return {"status": status, "response": response}

    def store(self, creditcard, options = None):
        raise NotImplementedError

    def unstore(self, identification, options = None):
        raise NotImplementedError



from django.utils.importlib import import_module
from django.conf import settings

gateway_cache = {}

class GatewayModuleNotFound(Exception):
    pass

class GatewayNotConfigured(Exception):
    pass

class CardNotSupported(Exception):
    pass

class InvalidData(Exception):
    pass

class Gateway(object):
    """Sub-classes to inherit from this and implement the below methods"""

    # To indicate if the gateway is in test mode or not
    test_mode = getattr(settings, "MERCHANT_TEST_MODE", True)

    # The below are optional attributes to be implemented and used by subclases.
    #
    # Set to indicate the default currency for the gateway.
    default_currency = ""
    # Sequence of countries supported by the gateway in ISO 3166 alpha-2 format.
    # http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2
    supported_countries = []
    # Sequence of supported card types by the gateway. Members should be valid 
    # subclasses of the Credit Card object.
    supported_cardtypes = []
    # Home page URL for the gateway. Used for information purposes only.
    homepage_url = ""
    # Name of the gateway.
    display_name = ""
    # Application name or some unique identifier for the gateway.
    application_id = ""

    def validate_card(self, credit_card):
        """Checks if the credit card is supported by the gateway
        and calls the `is_valid` method on it. Responsibility
        of the gateway author to use this method before every
        card transaction."""
        card_supported = None
        for card in self.supported_cardtypes:
            card_supported = card.regexp.match(credit_card.number)
            if card_supported:
                credit_card.card_type = card
                break
        if not card_supported:
            raise CardNotSupported("This credit card is not "
                                   "supported by the gateway.")
        # Gateways might provide some random number which
        # might not pass Luhn's test.
        if self.test_mode:
            return True 
        return credit_card.is_valid()

    def purchase(self, money, credit_card, options = None):
        """One go authorize and capture transaction"""
        raise NotImplementedError

    def authorize(self, money, credit_card, options = None):
        """Authorization for a future capture transaction"""
        raise NotImplementedError

    def capture(self, money, authorization, options = None):
        """Capture funds from a previously authorized transaction"""
        raise NotImplementedError

    def void(self, identification, options = None):
        """Null/Blank/Delete a previous transaction"""
        raise NotImplementedError

    def credit(self, money, identification, options = None):
        """Refund a previously 'settled' transaction"""
        raise NotImplementedError

    def recurring(self, money, creditcard, options = None):
        """Setup a recurring transaction"""
        raise NotImplementedError

    def store(self, creditcard, options = None):
        """Store the credit card and user profile information
        on the gateway for future use"""
        raise NotImplementedError

    def unstore(self, identification, options = None):
        """Delete the previously stored credit card and user
        profile information on the gateway"""
        raise NotImplementedError

def get_gateway(gateway, *args, **kwargs):
    """
    Return a gateway instance specified by `gateway` name.
    This caches gateway classes in a module-level dictionnary to avoid hitting
    the filesystem every time we require a gateway.
    
    Should the list of available gateways change at runtime, one should then 
    invalidate the cache, the simplest of ways would be to:
    
    >>> gateway_cache = {}
    """
    # Is the class in the cache?
    clazz = gateway_cache.get(gateway, None)
    
    if not clazz:
        # Let's actually load it (it's not in the cache)
        gateway_filename = "%s_gateway" %gateway
        gateway_module = None
        for app in settings.INSTALLED_APPS:
            try:
                gateway_module = import_module(".gateways.%s" %gateway_filename, package=app)
            except ImportError:
                pass
        if not gateway_module:
            raise GatewayModuleNotFound("Missing gateway")
        gateway_class_name = "".join(gateway_filename.title().split("_"))
        try:
            clazz = getattr(gateway_module, gateway_class_name)
        except AttributeError:
            raise GatewayNotConfigured("Missing %s class in the gateway module." %gateway_class_name)
        gateway_cache[gateway] = clazz
    # We either hit the cache or load our class object, let's return an instance
    # of it.
    return clazz(*args, **kwargs)




from django.conf import settings

from django.core.management.base import BaseCommand, CommandError

PAYPAL_STANDARD_IPN = ['PAYPAL_RECEIVER_EMAIL']
# PAYPAL_STANDARD_PDT = ['PAYPAL_IDENTITY_TOKEN']
PAYPAL_PRO = ['PAYPAL_TEST', 'PAYPAL_WPP_USER', 'PAYPAL_WPP_PASSWORD', 'PAYPAL_WPP_SIGNATURE']
AUTHORIZE = ['AUTHORIZE_LOGIN_ID', 'AUTHORIZE_TRANSACTION_KEY']
EWAY = ['EWAY_CUSTOMER_ID', 'EWAY_USERNAME', 'EWAY_PASSWORD']
GOOGLE_CHECKOUT = ['GOOGLE_CHECKOUT_MERCHANT_ID', 'GOOGLE_CHECKOUT_MERCHANT_KEY']

RBS_TEST = {'required': ['RBS_INSTALLTION_ID_TEST'], 'optional': ['RBS_MD5_SECRET_KEY']}
RBS_LIVE = {'required': ['RBS_INSTALLTION_ID_LIVE'], 'optional': ['RBS_MD5_SECRET_KEY']}

PAYMENT_GATEWAYS = {
    'authorize': AUTHORIZE,
    'eway': EWAY,
    'google_checkout': GOOGLE_CHECKOUT,
    'paypal_pro': PAYPAL_PRO,
    'paypal_ipn': PAYPAL_STANDARD_IPN,
    'rbs_test': RBS_TEST,
    'rbs_live': RBS_LIVE,
}

class Command(BaseCommand):
    help = 'Check for the required settings of billing app'
    args = PAYMENT_GATEWAYS.keys()
    def handle(self, *args, **kwargs):
        check_for_gateway = args or PAYMENT_GATEWAYS.keys()
        
        for gateway in check_for_gateway:
            if gateway not in PAYMENT_GATEWAYS:
                raise CommandError('Invalid payment gateway option %s, valid gateway options are %s' % (gateway, PAYMENT_GATEWAYS.keys()))
            required_settings = PAYMENT_GATEWAYS[gateway]
            
            if isinstance(required_settings, dict):
                if 'optional' in required_settings:
                    print '%s takes optional parameter %s' %(gateway, required_settings['optional'])
                required_settings = required_settings['required']
                
            
            for rs in required_settings:
                try:
                    getattr(settings, rs)
                except AttributeError:
                    # raising CommandError because the error message display is neat
                    raise CommandError('Missing parameter %s in settings for %s gateway' % (rs, gateway))
        return '0 errors'


"""
Template tags for Offsite payment gateways
"""
from django import template
from billing.templatetags.paypal_tags import paypal
from billing.templatetags.world_pay_tags import world_pay
from billing.templatetags.google_checkout_tags import google_checkout
from billing.templatetags.amazon_fps_tags import amazon_fps
from billing.templatetags.braintree_payments_tags import braintree_payments

register = template.Library()

register.tag(google_checkout)
register.tag(paypal)
register.tag(world_pay)
register.tag(amazon_fps)
register.tag(braintree_payments)




from django.db import models

class WorldPayResponse(models.Model):
    # merchant details
    installation_id = models.CharField(max_length=64)
    company_name    = models.CharField(max_length=255, blank=True, null=True)
    
    # purchase details sent by merchant
    cart_id     = models.CharField(max_length=255)
    description = models.TextField(blank=True)
    amount      = models.DecimalField(max_digits=16, decimal_places=2)
    currency    = models.CharField(max_length=64)
    
    # HTML string produced from the amount and currency 
    # that were submitted to initiate the payment
    amount_string = models.CharField(max_length=64)
    auth_mode = models.CharField(max_length=64)
    test_mode = models.CharField(max_length=64)
    
    # transaction details
    transaction_id     = models.CharField(max_length=64)
    transaction_status = models.CharField(max_length=64)
    transaction_time   = models.CharField(max_length=64)
    auth_amount        = models.DecimalField(max_digits=16, decimal_places=2)
    auth_currency      = models.CharField(max_length=64)
    auth_amount_string = models.CharField(max_length=64)
    raw_auth_message   = models.CharField(max_length=255)
    raw_auth_code      = models.CharField(max_length=64)
    
    # billing address of the user
    name = models.CharField(max_length=255)
    address      = models.TextField()
    post_code    = models.CharField(max_length=64)
    country_code = models.CharField(max_length=64)
    country      = models.CharField(max_length=64)
    phone        = models.CharField(u'Phone number', max_length=64, blank=True)
    fax          = models.CharField(u'Fax number', max_length=64, blank=True)
    email        = models.EmailField()
    
    # future pay id, for recurring payments
    future_pay_id = models.CharField(max_length=64, blank=True)
    
    card_type = models.CharField(max_length=64, blank=True)
    ip_address = models.IPAddressField(blank=True, null=True)

    class Meta:
        app_label = __name__.split(".")[0]



from django.test import TestCase
from billing.utils.credit_card import CreditCard
from billing import get_gateway

class GatewayTestCase(TestCase):
    def testCorrectClassLoading(self):
        gateway = get_gateway("pay_pal")
        self.assertEquals(gateway.display_name, "PayPal Website Payments Pro")



from django.db import models

class AmazonFPSResponse(models.Model):
    buyerEmail = models.EmailField()
    buyerName = models.CharField(max_length=75)
    callerReference = models.CharField(max_length=100)
    notificationType = models.CharField(max_length=50)
    operation = models.CharField(max_length=20)
    paymentMethod = models.CharField(max_length=5)
    recipientEmail = models.EmailField()
    recipientName = models.CharField(max_length=75)
    statusCode = models.CharField(max_length=50)
    statusMessage = models.TextField()
    # Because currency is sent along
    transactionAmount = models.CharField(max_length=20)
    transactionDate = models.DateTimeField()
    transactionId = models.CharField(max_length=50, db_index=True)
    transactionStatus = models.CharField(max_length=50)

    def __unicode__(self):
        return "%s : %s" %(self.transactionId, self.statusCode)

    class Meta:
        app_label = __name__.split(".")[0]



from django.utils.importlib import import_module
from django.conf import settings
from django.conf.urls.defaults import patterns

class IntegrationModuleNotFound(Exception):
    pass

class IntegrationNotConfigured(Exception):
    pass

integration_cache = {}

class Integration(object):
    """Base Integration class that needs to be subclassed by
    implementations"""
    # The mode of the gateway. Looks into the settings else
    # defaults to True
    test_mode = getattr(settings, "MERCHANT_TEST_MODE", True)

    def __init__(self, options=None):
        if not options:
            options = {}
        # The form fields that will be rendered in the template
        self.fields = {}
        self.fields.update(options)
    
    def add_field(self, key, value):
        self.fields[key] = value

    def add_fields(self, params):
        for (key, val) in params.iteritems():
            self.add_field(key, val)

    @property
    def service_url(self):
        # Modified by subclasses
        raise NotImplementedError

    def get_urls(self):
        # Method must be subclassed
        urlpatterns = patterns('')
        return urlpatterns

    @property
    def urls(self):
        return self.get_urls()

def get_integration(integration, *args, **kwargs):
    """Return a integration instance specified by `integration` name"""

    klass = integration_cache.get(integration, None)

    if not klass:
        integration_filename = "%s_integration" %integration
        integration_module = None
        for app in settings.INSTALLED_APPS:
            try:
                integration_module = import_module(".integrations.%s" %integration_filename, package=app)
            except ImportError:
                pass
        if not integration_module:
            raise IntegrationModuleNotFound("Missing integration")
        integration_class_name = "".join(integration_filename.title().split("_"))
        try:
            klass = getattr(integration_module, integration_class_name)
        except AttributeError:
            raise IntegrationNotConfigured("Missing %s class in the integration module." %integration_class_name)
        integration_cache[integration] = klass
    return klass(*args, **kwargs)



from billing import Integration
from django.conf import settings
from django.views.decorators.http import require_GET
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from django.conf.urls.defaults import patterns, url
import braintree, urllib
from django.core.urlresolvers import reverse
from billing.forms.braintree_payments_forms import BraintreePaymentsForm
from django.shortcuts import render_to_response
from django.template import RequestContext

class BraintreePaymentsIntegration(Integration):
    def __init__(self, options=None):
        if not options:
            options = {}
        super(BraintreePaymentsIntegration, self).__init__(options=options)

        test_mode = getattr(settings, "MERCHANT_TEST_MODE", True)
        if test_mode:
            env = braintree.Environment.Sandbox
        else:
            env = braintree.Environment.Production
        braintree.Configuration.configure(
            env,
            settings.BRAINTREE_MERCHANT_ACCOUNT_ID,
            settings.BRAINTREE_PUBLIC_KEY,
            settings.BRAINTREE_PRIVATE_KEY
            )

    @property
    def service_url(self):
        return braintree.TransparentRedirect.url()

    def braintree_notify_handler(self, request):
        fpath = request.get_full_path()
        query_string = fpath.split("?", 1)[1]
        result = braintree.TransparentRedirect.confirm(query_string)
        if result.is_success:
            transaction_was_successful.send(sender=self,
                                            type="sale",
                                            response=result)
            return self.braintree_success_handler(request, result)
        transaction_was_unsuccessful.send(sender=self,
                                          type="sale",
                                          response=result)
        return self.braintree_failure_handler(request, result)

    def braintree_success_handler(self, request, response):
        return render_to_response("billing/braintree_success.html", 
                                  {"response": response},
                                  context_instance=RequestContext(request))

    def braintree_failure_handler(self, request, response):
        return render_to_response("billing/braintree_failure.html", 
                                  {"response": response},
                                  context_instance=RequestContext(request))

    def get_urls(self):
        urlpatterns = patterns('',
           url('^braintree-notify-handler/$', self.braintree_notify_handler, name="braintree_notify_handler"),)
        return urlpatterns

    def add_fields(self, params):
        for (key, val) in params.iteritems():
            if isinstance(val, dict):
                new_params = {}
                for k in val:
                    new_params["%s__%s" %(key, k)] = val[k]
                self.add_fields(new_params)
            else:
                self.add_field(key, val)

    def generate_tr_data(self):
        tr_data_dict = {"transaction": {}}
        tr_data_dict["transaction"]["type"] = self.fields["transaction__type"]
        tr_data_dict["transaction"]["order_id"] = self.fields["transaction__order_id"]
        if self.fields.get("transaction__customer_id"):
            tr_data_dict["transaction"]["customer_id"] = fields["transaction__customer__id"]
        if self.fields.get("transaction__customer__id"):
            tr_data_dict["transaction"]["customer"] = {"id": self.fields["transaction__customer__id"]}
        tr_data_dict["transaction"]["options"] = {"submit_for_settlement": 
                                                  self.fields.get("transaction__options__submit_for_settlement", True)}
        if self.fields.get("transaction__payment_method_token"):
            tr_data_dict["transaction"]["payment_method_token"] = self.fields["transaction__payment_method_token"]
        if self.fields.get("transaction__credit_card__token"):
            tr_data_dict["transaction"]["credit_card"] = {"token": self.fields["transaction__credit_card__token"]}
        if self.fields.get("transaction__amount"):
            tr_data_dict["transaction"]["amount"] = self.fields["transaction__amount"]
        notification_url = "%s%s" %(self.fields["site"], reverse("braintree_notify_handler"))
        tr_data = braintree.Transaction.tr_data_for_sale(tr_data_dict, notification_url)
        return tr_data

    def generate_form(self):
        initial_data = self.fields
        initial_data.update({"tr_data": self.generate_tr_data()})
        form = BraintreePaymentsForm(initial=initial_data)
        return form




from django.conf import settings
from eway_api.client import RebillEwayClient, HOSTED_TEST_URL, HOSTED_LIVE_URL
from billing import Gateway
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from billing.utils.credit_card import Visa, MasterCard, DinersClub, JCB, AmericanExpress

class EwayGateway(Gateway):
    default_currency = "AUD"
    supported_countries = ["AU"]
    supported_cardtypes = [Visa, MasterCard, AmericanExpress, DinersClub, JCB]
    homepage_url = "https://eway.com.au/"
    display_name = "eWay"

    def __init__(self):
        self.test_mode = getattr(settings, 'MERCHANT_TEST_MODE', True)
        self.client = RebillEwayClient(test_mode=self.test_mode,
                                      customer_id=settings.EWAY_CUSTOMER_ID,
                                      username=settings.EWAY_USERNAME,
                                      password=settings.EWAY_PASSWORD,
                                      url=self.service_url,
                                      )
        self.hosted_customer = self.client.client.factory.create("CreditCard")
    
    def add_creditcard(self, credit_card):
        """add credit card details to the request parameters"""
        self.hosted_customer.CCNumber = credit_card.number
        self.hosted_customer.CCNameOnCard = credit_card.name
        self.hosted_customer.CCExpiryMonth = '%02d' % (credit_card.month)
        self.hosted_customer.CCExpiryYear = str(credit_card.year)[-2:]
        self.hosted_customer.FirstName = credit_card.first_name
        self.hosted_customer.LastName = credit_card.last_name
    
    def add_address(self, options=None):
        """add address details to the request parameters"""
        if not options:
            options = {}
        address = options.get("billing_address", {})
        self.hosted_customer.Title = address.get("salutation", "Mr./Ms.")
        self.hosted_customer.Address = address.get("address1", '') + address.get("address2", "")
        self.hosted_customer.Suburb = address.get("city")
        self.hosted_customer.State = address.get("state")
        self.hosted_customer.Company = address.get("company")
        self.hosted_customer.PostCode = address.get("zip")
        self.hosted_customer.Country = address.get("country")
        self.hosted_customer.Email = options.get("email")
        self.hosted_customer.Fax = address.get("fax")
        self.hosted_customer.Phone = address.get("phone")
        self.hosted_customer.Mobile = address.get("mobile")
        self.hosted_customer.CustomerRef = address.get("customer_ref")
        self.hosted_customer.JobDesc = address.get("job_desc")
        self.hosted_customer.Comments = address.get("comments")
        self.hosted_customer.URL = address.get("url")

    @property
    def service_url(self):
        if self.test_mode:
            return HOSTED_TEST_URL
        return HOSTED_LIVE_URL

    def purchase(self, money, credit_card, options=None):
        """Using Eway payment gateway , charge the given
        credit card for specified money"""
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")
        self.add_creditcard(credit_card)
        self.add_address(options)
        
        customer_id = self.client.create_hosted_customer(self.hosted_customer)
        if self.test_mode:
            customer_id = getattr(settings, 'EWAY_TEST_CUSTOMER_ID')
        pymt_response = self.client.process_payment(customer_id, 
                                                    money, 
                                                    options.get("invoice", 'test'),
                                                    options.get("description", 'test'))
        
        if not hasattr(pymt_response, "ewayTrxnStatus"):
            transaction_was_unsuccessful.send(sender=self,
                                              type="purchase",
                                              response=pymt_response)
            return {"status": "FAILURE", "response": pymt_response}

        if pymt_response.ewayTrxnStatus == "False":
            transaction_was_unsuccessful.send(sender=self,
                                              type="purchase",
                                              response=pymt_response)
            return {"status": "FAILURE", "response": pymt_response}

        transaction_was_successful.send(sender=self,
                                        type="purchase",
                                        response=pymt_response)
        return {"status": "SUCCESS", "response": pymt_response}
    
    def authorize(self, money, credit_card, options = None):
        raise NotImplementedError

    def capture(self, money, authorization, options = None):
        raise NotImplementedError

    def void(self, identification, options = None):
        raise NotImplementedError

    def credit(self, money, identification, options = None):
        raise NotImplementedError

    def recurring(self, money, creditcard, options = None):
        raise NotImplementedError

    def store(self, creditcard, options = None):
        raise NotImplementedError

    def unstore(self, identification, options = None):
        raise NotImplementedError




from django import forms
from django.conf import settings
import braintree

class BraintreePaymentsForm(forms.Form):
    transaction__customer__first_name = forms.CharField(max_length=50, required=False)
    transaction__customer__last_name  = forms.CharField(max_length=50, required=False)
    transaction__customer__company    = forms.CharField(max_length=100, required=False)
    transaction__customer__email      = forms.EmailField(required=False)
    transaction__customer__phone      = forms.CharField(max_length=15, required=False)
    transaction__customer__fax        = forms.CharField(max_length=15, required=False)
    transaction__customer__website    = forms.URLField(verify_exists=True, required=False)
    transaction__credit_card__cardholder_name = forms.CharField(max_length=100)
    transaction__credit_card__number  = forms.CharField()
    transaction__credit_card__cvv     = forms.CharField(max_length=4)
    transaction__credit_card__expiration_date = forms.CharField(max_length=7)

    transaction__billing__first_name = forms.CharField(max_length=50, required=False)
    transaction__billing__last_name  = forms.CharField(max_length=50, required=False)
    transaction__billing__company    = forms.CharField(max_length=100, required=False)
    transaction__billing__street_address = forms.CharField(widget=forms.Textarea(), required=False)
    transaction__billing__extended_address = forms.CharField(widget=forms.Textarea(), required=False)
    transaction__billing__locality = forms.CharField(max_length=50, required=False)
    transaction__billing__region   = forms.CharField(max_length=50, required=False)
    transaction__billing__postal_code = forms.CharField(max_length=10, required=False)
    transaction__billing__country_code_alpha2 = forms.CharField(max_length=2, required=False)
    transaction__billing__country_code_alpha3 = forms.CharField(max_length=3, required=False)
    transaction__billing__country_code_numeric = forms.IntegerField(required=False, min_value=0)
    transaction__billing__country_name = forms.CharField(max_length=50, required=False)

    transaction__shipping__first_name = forms.CharField(max_length=50, required=False)
    transaction__shipping__last_name = forms.CharField(max_length=50, required=False)
    transaction__shipping__company = forms.CharField(max_length=100, required=False)
    transaction__shipping__street_address = forms.CharField(widget=forms.Textarea(), required=False)
    transaction__shipping__extended_address = forms.CharField(widget=forms.Textarea(), required=False)
    transaction__shipping__locality = forms.CharField(max_length=50, required=False)
    transaction__shipping__region   = forms.CharField(max_length=50, required=False)
    transaction__shipping__postal_code = forms.CharField(max_length=10, required=False)
    transaction__shipping__country_code_alpha2 = forms.CharField(max_length=2, required=False)
    transaction__shipping__country_code_alpha3 = forms.CharField(max_length=3, required=False)
    transaction__shipping__country_code_numeric = forms.IntegerField(required=False, min_value=0)
    transaction__shipping__country_name = forms.CharField(max_length=50, required=False)

    transaction__options__add_billing_address_to_payment_method = forms.BooleanField(required=False)
    transaction__options__store_shipping_address_in_vault       = forms.BooleanField(required=False)
    transaction__options__store_in_vault_on_success             = forms.BooleanField(required=False)
    transaction__options__submit_for_settlement = forms.BooleanField(required=False)

    transaction__type            = forms.CharField(max_length=10)
    transaction__amount          = forms.DecimalField(required=False)
    transaction__order_id        = forms.CharField(max_length=50)
    transaction__customer__id    = forms.CharField(max_length=50, required=False)
    transaction__credit_card__token = forms.CharField(max_length=50, required=False)
    transaction__payment_method_token = forms.CharField(max_length=50, required=False)

    tr_data = forms.CharField(widget=forms.HiddenInput())



from django.test import TestCase
from django.utils.html import strip_spaces_between_tags
from billing import get_integration
from django.template import Template, Context
from django.conf import settings

class WorldPayTestCase(TestCase):
    def setUp(self):
        self.wp = get_integration("world_pay")
        fields = {
            "cartId": "TEST123",
            "amount": 1,
            "currency": "USD",
            "testMode": 100,
            "futurePayType": "regular",
            "option": 0,
            "noOfPayments": 12,
            "intervalUnit": 3,
            "intervalMult": 1,
            "normalAmount": 1,
            "startDelayUnit": 3,
            "startDelayMult": 1,
            "instId": 12345,
            "signatureFields": "instId:amount:cartId",
            }
        self.wp.add_fields(fields)

    def testFormGen(self):
        # Since the secret key cannot be distributed
        settings.WORLDPAY_MD5_SECRET_KEY = "test"
        tmpl = Template("{% load billing_tags %}{% world_pay obj %}")
        form = tmpl.render(Context({"obj": self.wp}))
        pregen_form = """<form method='post' action='https://select-test.wp3.rbsworldpay.com/wcc/purchase'><input type="hidden" name="futurePayType" value="regular" id="id_futurePayType" /><input type="hidden" name="intervalUnit" value="3" id="id_intervalUnit" /><input type="hidden" name="intervalMult" value="1" id="id_intervalMult" /><input type="hidden" name="option" value="0" id="id_option" /><input type="hidden" name="noOfPayments" value="12" id="id_noOfPayments" /><input type="hidden" name="normalAmount" value="1" id="id_normalAmount" /><input type="hidden" name="startDelayUnit" value="3" id="id_startDelayUnit" /><input type="hidden" name="startDelayMult" value="1" id="id_startDelayMult" /><input type="hidden" name="instId" value="12345" id="id_instId" /><input type="hidden" name="cartId" value="TEST123" id="id_cartId" /><input type="hidden" name="amount" value="1" id="id_amount" /><input type="hidden" name="currency" value="USD" id="id_currency" /><input type="hidden" name="desc" id="id_desc" /><input type="hidden" name="testMode" value="100" id="id_testMode" /><input type="hidden" name="signatureFields" value="instId:amount:cartId" id="id_signatureFields" /><input type="hidden" name="signature" value="75be85df059de95a3407334b7d21273c" id="id_signature" /><input type='submit' value='Pay through WorldPay'/></form>"""
        self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())

    def testFormGen2(self):
        # Since the secret key cannot be distributed
        self.wp.add_field("signatureFields", "instId:amount:currency:cartId")
        self.wp.fields.pop("signature", None)
        settings.WORLDPAY_MD5_SECRET_KEY = "test"
        tmpl = Template("{% load billing_tags %}{% world_pay obj %}")
        form = tmpl.render(Context({"obj": self.wp}))
        pregen_form = """<form method='post' action='https://select-test.wp3.rbsworldpay.com/wcc/purchase'><input type="hidden" name="futurePayType" value="regular" id="id_futurePayType" /><input type="hidden" name="intervalUnit" value="3" id="id_intervalUnit" /><input type="hidden" name="intervalMult" value="1" id="id_intervalMult" /><input type="hidden" name="option" value="0" id="id_option" /><input type="hidden" name="noOfPayments" value="12" id="id_noOfPayments" /><input type="hidden" name="normalAmount" value="1" id="id_normalAmount" /><input type="hidden" name="startDelayUnit" value="3" id="id_startDelayUnit" /><input type="hidden" name="startDelayMult" value="1" id="id_startDelayMult" /><input type="hidden" name="instId" value="12345" id="id_instId" /><input type="hidden" name="cartId" value="TEST123" id="id_cartId" /><input type="hidden" name="amount" value="1" id="id_amount" /><input type="hidden" name="currency" value="USD" id="id_currency" /><input type="hidden" name="desc" id="id_desc" /><input type="hidden" name="testMode" value="100" id="id_testMode" /><input type="hidden" name="signatureFields" value="instId:amount:currency:cartId" id="id_signatureFields" /><input type="hidden" name="signature" value="9be5c6334f058a294b1ea352634a3dd7" id="id_signature" /><input type='submit' value='Pay through WorldPay'/></form>"""
        self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())



from billing import Gateway
from billing.gateway import InvalidData
from billing.signals import *
from billing.utils.credit_card import InvalidCard, Visa, MasterCard, \
    AmericanExpress, Discover
from django.conf import settings
import braintree

class BraintreePaymentsGateway(Gateway):
    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    supported_countries = ["US"]
    default_currency = "USD"

    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    homepage_url = "http://www.braintreepayments.com/"
    display_name = "Braintree Payments"

    def __init__(self):
        test_mode = getattr(settings, "MERCHANT_TEST_MODE", True)
        if test_mode:
            env = braintree.Environment.Sandbox
        else:
            env = braintree.Environment.Production
        braintree.Configuration.configure(
            env,
            settings.BRAINTREE_MERCHANT_ACCOUNT_ID,
            settings.BRAINTREE_PUBLIC_KEY,
            settings.BRAINTREE_PRIVATE_KEY
            )

    def _cc_expiration_date(self, credit_card):
        return "%s/%s" %(credit_card.month, credit_card.year)

    def _cc_cardholder_name(self, credit_card):
        return "%s %s" %(credit_card.first_name, credit_card.last_name)

    def _build_request_hash(self, options):
        request_hash = {
                "order_id": options.get("order_id", ""),
                }
        if options.get("customer"):
            name = options["customer"].get("name", "")
            try:
                first_name, last_name = name.split(" ", 1)
            except ValueError:
                first_name = name
                last_name = ""
            request_hash.update({
                "customer": {
                    "first_name": first_name,
                    "last_name": last_name,
                    "company": options["customer"].get("company", ""),
                    "phone": options["customer"].get("phone", ""),
                    "fax": options["customer"].get("fax", ""),
                    "website": options["customer"].get("website", ""),
                    "email": options["customer"].get("email", options.get("email", ""))
                    }
                })
        if options.get("billing_address"):
            name = options["billing_address"].get("name", "")
            try:
                first_name, last_name = name.split(" ", 1)
            except ValueError:
                first_name = name
                last_name = ""
            request_hash.update({
                "billing": {
                    "first_name": first_name,
                    "last_name": last_name,
                    "company": options["billing_address"].get("company", ""),
                    "street_address": options["billing_address"].get("address1", ""),
                    "extended_address": options["billing_address"].get("address2", ""),
                    "locality": options["billing_address"].get("city", ""),
                    "region": options["billing_address"].get("state", ""),
                    "postal_code": options["billing_address"].get("zip", ""),
                    "country_code_alpha2": options["billing_address"].get("country", "")
                    }
                })
        if options.get("shipping_address"):
            name = options["shipping_address"].get("name", "")
            try:
                first_name, last_name = name.split(" ", 1)
            except ValueError:
                first_name = name
                last_name = ""
            request_hash.update({
                "shipping": {
                    "first_name": first_name,
                    "last_name": last_name,
                    "company": options["shipping_address"].get("company", ""),
                    "street_address": options["shipping_address"].get("address1", ""),
                    "extended_address": options["shipping_address"].get("address2", ""),
                    "locality": options["shipping_address"].get("city", ""),
                    "region": options["shipping_address"].get("state", ""),
                    "postal_code": options["shipping_address"].get("zip", ""),
                    "country_code_alpha2": options["shipping_address"].get("country", "")
                    }
                })
        return request_hash

    def purchase(self, money, credit_card, options = None):
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")

        request_hash = self._build_request_hash(options)
        request_hash["amount"] = money
        request_hash["credit_card"] = {
            "number": credit_card.number,
            "expiration_date": self._cc_expiration_date(credit_card),
            "cardholder_name": self._cc_cardholder_name(credit_card),
            "cvv": credit_card.verification_value,
            }
        braintree_options = options.get("options", {})
        braintree_options.update({"submit_for_settlement": True})
        request_hash.update({
                "options": braintree_options
                })
        response = braintree.Transaction.sale(request_hash)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self, 
                                            type="purchase",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="purchase",
                                              response=response)
        return {"status": status, "response": response}

    def authorize(self, money, credit_card, options = None):
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")

        request_hash = self._build_request_hash(options)
        request_hash["amount"] = money
        request_hash["credit_card"] = {
            "number": credit_card.number,
            "expiration_date": self._cc_expiration_date(credit_card),
            "cardholder_name": self._cc_cardholder_name(credit_card),
            "cvv": credit_card.verification_value,
            }
        braintree_options = options.get("options", {})
        if braintree_options:
            request_hash.update({
                    "options": braintree_options
                    })
        response = braintree.Transaction.sale(request_hash)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self, 
                                            type="authorize",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="authorize",
                                              response=response)
        return {"status": status, "response": response}

    def capture(self, money, authorization, options = None):
        response = braintree.Transaction.submit_for_settlement(authorization, money)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="capture",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="capture",
                                              response=response)
        return {"status": status, "response": response}

    def void(self, identification, options = None):
        response = braintree.Transaction.void(identification)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="void",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="void",
                                              response=response)
        return {"status": status, "response": response}

    def credit(self, money, identification, options = None):
        response = braintree.Transaction.refund(identification, money)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="credit",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="credit",
                                              response=response)
        return {"status": status, "response": response}

    def recurring(self, money, credit_card, options = None):
        resp = self.store(credit_card, options = options)
        if resp["status"] == "FAILURE":
            transaction_was_unsuccessful.send(sender=self,
                                              type="recurring",
                                              response=resp)
            return resp
        payment_token = resp["response"].customer.credit_cards[0].token
        request_hash = options["recurring"]
        request_hash.update({
            "payment_method_token": payment_token,
            })
        response = braintree.Subscription.create(request_hash)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="recurring",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="recurring",
                                              response=response)
        return {"status": status, "response": response}

    def store(self, credit_card, options = None):
        if not options:
            options = {}

        customer = options.get("customer", None)
        if not customer:
            raise InvalidData("Customer information needs to be passed.")

        try:
            first_name, last_name = customer["name"].split(" ", 1)
        except ValueError:
            first_name = customer["name"]
            last_name = ""

        search_resp = braintree.Customer.search(
            braintree.CustomerSearch.cardholder_name == credit_card.name,
            braintree.CustomerSearch.credit_card_number.starts_with(credit_card.number[:6]),
            braintree.CustomerSearch.credit_card_number.ends_with(credit_card.number[-4:]),
            braintree.CustomerSearch.credit_card_expiration_date == self._cc_expiration_date(credit_card)
            )

        customer_list = []
        for customer in search_resp.items:
            customer_list.append(customer)

        if len(customer_list) >= 1:
            # Take the first customer
            customer = customer_list[0]
        else:
            card_hash = {
                "number": credit_card.number,
                "expiration_date": self._cc_expiration_date(credit_card),
                "cardholder_name": self._cc_cardholder_name(credit_card),
                }

            if options.get("options"):
                card_hash["options"] = options["options"]

            request_hash = {
                "first_name": first_name,
                "last_name": last_name,
                "company": customer.get("company", ""),
                "email": customer.get("email", options.get("email", "")),
                "phone": customer.get("phone", ""),
                "credit_card": card_hash, 
                }
            result = braintree.Customer.create(request_hash)
            if not result.is_success: 
                transaction_was_unsuccessful.send(sender=self,
                                                  type="store",
                                                  response=result)
                return {"status": "FAILURE", "response": result}
            customer = result.customer

        request_hash = {}
        if options.get("billing_address"):
            name = options["billing_address"].get("name", "")
            try:
                first_name, last_name = name.split(" ", 1)
            except ValueError:
                first_name = name
                last_name = ""

            request_hash.update({
                "first_name": first_name,
                "last_name": last_name,
                "company": options.get("company", ""),
                "street_address": options.get("address1", ""),
                "extended_address": options.get("address2", ""),
                "locality": options.get("city", ""),
                "region": options.get("state", ""),
                "postal_code": options.get("zip", ""),
                "country_name": options.get("country", "")
                })

        card_hash = {
            "number": credit_card.number,
            "expiration_date": self._cc_expiration_date(credit_card),
            "cardholder_name": self._cc_cardholder_name(credit_card),
            "options": {
                "update_existing_token": customer.credit_cards[0].token,
                }
            }
        if options.get("options"):
            card_hash["options"].update(options["options"])
        if request_hash:
            card_hash.update({"billing_address": request_hash})
        response = braintree.Customer.update(customer.id, {
                "credit_card": card_hash,
                })
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="store",
                                            response=response)
        else:
            for ii in response.errors.deep_errors:
                print ii.message
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="store",
                                              response=response)
        return {"status": status, "response": response}
            
    def unstore(self, identification, options = None):
        response = braintree.CreditCard.delete(identification)
        if response.is_success:
            status = "SUCCESS"
            transaction_was_successful.send(sender=self,
                                            type="unstore",
                                            response=response)
        else:
            status = "FAILURE"
            transaction_was_unsuccessful.send(sender=self,
                                              type="unstore",
                                              response=response)
        return {"status": status, "response": response}



import unittest
from client import RebillEwayClient, HOSTED_TEST_URL
from suds import WebFault

# uncomment to enable debugging
#import logging
#logging.basicConfig(level=logging.DEBUG)
#logging.getLogger('suds.client').setLevel(logging.DEBUG)

class ClientTestCase(unittest.TestCase):
    def setUp(self):
        self.rebill_test = RebillEwayClient(test_mode=True, customer_id='87654321', username='test@eway.com.au', password='test123')
        self.rebill_customer = self.rebill_test.client.factory.create("CustomerDetails")
        self.rebill_event = self.rebill_test.client.factory.create("RebillEventDetails")
        self.hosted_test = RebillEwayClient(test_mode=True, 
                                            customer_id='87654321', 
                                            username='test@eway.com.au', 
                                            password='test123', 
                                            url=HOSTED_TEST_URL)
        self.hosted_customer = self.hosted_test.client.factory.create("CreditCard")
        
    def test_create_rebill_customer(self):
        self.rebill_customer.CustomerTitle = "Mr."
        self.rebill_customer.CustomerFirstName = "Joe"
        self.rebill_customer.CustomerLastName = "Bloggs"
        self.rebill_customer.CustomerAddress = "test street"
        self.rebill_customer.CustomerSuburb = "Sydney"
        self.rebill_customer.CustomerState = "NSW"
        self.rebill_customer.CustomerCompany = "Test Company"
        self.rebill_customer.CustomerPostCode = "2000" 
        self.rebill_customer.CustomerCountry = "au"
        self.rebill_customer.CustomerEmail = "test@eway.com.au"
        self.rebill_customer.CustomerFax = "0267720000"
        self.rebill_customer.CustomerPhone1 = "0267720000" 
        self.rebill_customer.CustomerPhone2 = "0404085992" 
        self.rebill_customer.CustomerRef = "REF100"
        self.rebill_customer.CustomerJobDesc = "test"
        self.rebill_customer.CustomerComments = "Now!"
        self.rebill_customer.CustomerURL = "http://www.google.com.au"
        
        new_rebill_customer = self.rebill_test.create_rebill_customer(self.rebill_customer)
        print "create rebill customer", new_rebill_customer
        self.assertEqual(new_rebill_customer.Result, "Success")
        
    def test_create_rebill_customer_with_kwargs(self):
        new_rebill_customer_with_kwargs = self.rebill_test.create_rebill_customer(
                                                                           customerTitle="Mr.", 
                                                                           customerFirstName="Joe", 
                                                                           customerLastName="Bloggs", 
                                                                           customerAddress="test street", 
                                                                           customerSuburb="Sydney", 
                                                                           customerState="NSW", 
                                                                           customerCompany="Test Company", 
                                                                           customerPostCode="2000", 
                                                                           customerCountry="au", 
                                                                           customerEmail="test@eway.com.au", 
                                                                           customerFax="0267720000", 
                                                                           customerPhone1="0267720000", 
                                                                           customerPhone2="0404085992", 
                                                                           customerRef="REF100", 
                                                                           customerJobDesc="test",
                                                                           customerURL="http://www.google.com.au",
                                                                           customerComments="Now!",
                                                                           )
        print "create rebill customer with kwargs", new_rebill_customer_with_kwargs
        self.assertEqual(new_rebill_customer_with_kwargs.Result, "Success")
        
    def test_update_rebill_customer(self):
        updated_rebill_customer = self.rebill_test.update_rebill_customer(
                                                                               RebillCustomerID="17609", 
                                                                               customerTitle="Mr.", 
                                                                               customerFirstName="Joe", 
                                                                               customerLastName="Bloggs", 
                                                                               customerAddress="test street", 
                                                                               customerSuburb="Sydney", 
                                                                               customerState="NSW", 
                                                                               customerCompany="Test Company", 
                                                                               customerPostCode="2000", 
                                                                               customerCountry="au", 
                                                                               customerEmail="test@eway.com.au", 
                                                                               customerFax="0267720000", 
                                                                               customerPhone1="0267720000", 
                                                                               customerPhone2="0404085992", 
                                                                               customerRef="REF100", 
                                                                               customerJobDesc="test",
                                                                               customerURL="http://www.google.com.au",
                                                                               customerComments="Now!",                                                                                
                                                                               )
        print "update rebill customer", updated_rebill_customer
        self.assertEqual(updated_rebill_customer.Result, "Success")
        
    def test_delete_rebill_customer(self):
        deleted_rebill_customer = self.rebill_test.delete_rebill_customer("10292")
        print "delete rebill customer", deleted_rebill_customer
        self.assertEqual(deleted_rebill_customer.Result, "Success")
        
    def test_create_rebill_event(self):
        self.rebill_event.RebillCustomerID = "60001545"
        self.rebill_event.RebillID = ""
        self.rebill_event.RebillInvRef = "ref123"
        self.rebill_event.RebillInvDesc = "test event"
        self.rebill_event.RebillCCName = "test"
        self.rebill_event.RebillCCNumber = "4444333322221111"
        self.rebill_event.RebillCCExpMonth = "07"
        self.rebill_event.RebillCCExpYear = "12"
        self.rebill_event.RebillInitAmt = "100"
        self.rebill_event.RebillInitDate = "08/06/2011"
        self.rebill_event.RebillRecurAmt = "100"
        self.rebill_event.RebillStartDate = "09/06/2011"
        self.rebill_event.RebillInterval = "1"
        self.rebill_event.RebillIntervalType = "1"
        self.rebill_event.RebillEndDate = "08/07/2011"
        
        new_rebill_event = self.rebill_test.create_rebill_event(self.rebill_event)
        print "create rebill event", new_rebill_event
        self.assertEqual(new_rebill_event.Result, "Success")
        
    def test_create_rebill_event_with_kwargs(self):
        new_rebill_event_with_kwargs = self.rebill_test.create_rebill_event(
                                                                                 RebillCustomerID="60001545",                                                                                 
                                                                                 RebillInvRef="ref123", 
                                                                                 RebillInvDes="test", 
                                                                                 RebillCCName="test", 
                                                                                 RebillCCNumber="4444333322221111", 
                                                                                 RebillCCExpMonth="07", 
                                                                                 RebillCCExpYear="12", 
                                                                                 RebillInitAmt="100", 
                                                                                 RebillInitDate="08/06/2011", 
                                                                                 RebillRecurAmt="100", 
                                                                                 RebillStartDate="09/06/2011", 
                                                                                 RebillInterval="1", 
                                                                                 RebillIntervalType="1", 
                                                                                 RebillEndDate="08/07/2011"                                                                                 
                                                                                 )
        print "create rebill event with kwargs", new_rebill_event_with_kwargs
        self.assertEqual(new_rebill_event_with_kwargs.Result, "Success")
    
    def test_update_rebill_event(self):
        updated_rebill_event = self.rebill_test.update_rebill_event(
                                                                         RebillCustomerID="60001545", 
                                                                         RebillID="80001208", 
                                                                         RebillInvRef="ref123", 
                                                                         RebillInvDes="test", 
                                                                         RebillCCName="test", 
                                                                         RebillCCNumber="4444333322221111", 
                                                                         RebillCCExpMonth="07", 
                                                                         RebillCCExpYear="12", 
                                                                         RebillInitAmt="100", 
                                                                         RebillInitDate="08/06/2011", 
                                                                         RebillRecurAmt="100", 
                                                                         RebillStartDate="09/06/2011", 
                                                                         RebillInterval="1", 
                                                                         RebillIntervalType="1", 
                                                                         RebillEndDate="08/07/2011"
                                                                         )
        print "update rebill event", updated_rebill_event
        self.assertEqual(updated_rebill_event.Result, "Success")
                
    def test_delete_rebill_event(self):
        deleted_rebill_event = self.rebill_test.delete_rebill_event("10292", "80001208")
        print "delete rebill event", deleted_rebill_event
        self.assertEqual(deleted_rebill_event.Result, "Success")
        
    def test_query_next_transaction(self):
        query_next_transaction_result = self.rebill_test.query_next_transaction("60001545", "80001227")
        print "test_query_next_transaction", query_next_transaction_result
        self.assertFalse(query_next_transaction_result == None)
        
    def test_query_rebill_customer(self):
        query_rebill_customer_result = self.rebill_test.query_rebill_customer("60001545")
        print "test_query_rebill_customer", query_rebill_customer_result
        self.assertFalse(query_rebill_customer_result == None)
        
    def test_query_rebill_event(self):
        query_rebill_result = self.rebill_test.query_rebill_event("60001545", "80001227")
        print "test_query_rebill_event", query_rebill_result
        self.assertFalse(query_rebill_result == None)
        
    def test_query_transactions(self):
        query_transactions_result = self.rebill_test.query_transactions("60001545", "80001208")
        print "test_query_transactions", query_transactions_result
        self.assertFalse(query_transactions_result == None)
        
    def test_create_hosted_customer(self):
        self.hosted_customer.Title = "Mr."
        self.hosted_customer.FirstName = "Joe"
        self.hosted_customer.LastName = "Bloggs"
        self.hosted_customer.Address = "test street"
        self.hosted_customer.Suburb = "Sydney"
        self.hosted_customer.State = "NSW"
        self.hosted_customer.Company = "Test Company"
        self.hosted_customer.PostCode = "2000"
        self.hosted_customer.Country = "au"
        self.hosted_customer.Email = "test@eway.com.au"
        self.hosted_customer.Fax = "0267720000"
        self.hosted_customer.Phone = "0267720000"
        self.hosted_customer.Mobile = "0404085992"
        self.hosted_customer.CustomerRef = "REF100"
        self.hosted_customer.JobDesc = "test"
        self.hosted_customer.Comments = "Now!"
        self.hosted_customer.URL = "http://www.google.com.au"
        self.hosted_customer.CCNumber = "4444333322221111"
        self.hosted_customer.CCNameOnCard = "test"
        self.hosted_customer.CCExpiryMonth = "07"
        self.hosted_customer.CCExpiryYear = "12"

        new_hosted_customer_id = self.hosted_test.create_hosted_customer(self.hosted_customer)
        print "create new hosted customer", new_hosted_customer_id
        self.assertFalse(isinstance(new_hosted_customer_id, WebFault))
        
    def test_create_hosted_customer_with_kwargs(self):
        new_hosted_customer_id = self.hosted_test.create_hosted_customer(
                                                                          Title="Mr.", 
                                                                          FirstName="Joe", 
                                                                          LastName="Bloggs", 
                                                                          Address="test street", 
                                                                          Suburb="Sydney", 
                                                                          State="NSW", 
                                                                          Company="Test Company", 
                                                                          PostCode="2000", 
                                                                          Country="au", 
                                                                          Email="test@eway.com.au", 
                                                                          Fax="0267720000", 
                                                                          Phone="0267720000", 
                                                                          Mobile="0404085992", 
                                                                          CustomerRef="REF100", 
                                                                          JobDesc="test", 
                                                                          Comments="Now!", 
                                                                          URL="http://www.google.com.au", 
                                                                          CCNumber="4444333322221111", 
                                                                          CCNameOnCard="test", 
                                                                          CCExpiryMonth="07", 
                                                                          CCExpiryYear="12"
                                                                          )
        print "create new hosted customer with kwargs", new_hosted_customer_id
        self.assertFalse(isinstance(new_hosted_customer_id, WebFault))
        
    def test_update_hosted_customer(self):
        updated_hosted_customer = self.hosted_test.update_hosted_customer(
                                                                          managedCustomerID="9876543211000",
                                                                          Title="Mr.", 
                                                                          FirstName="Joe", 
                                                                          LastName="Bloggs", 
                                                                          Address="test street", 
                                                                          Suburb="Sydney", 
                                                                          State="NSW", 
                                                                          Company="Test Company", 
                                                                          PostCode="2000", 
                                                                          Country="au", 
                                                                          Email="test@eway.com.au", 
                                                                          Fax="0267720000", 
                                                                          Phone="0267720000", 
                                                                          Mobile="0404085992", 
                                                                          CustomerRef="REF100", 
                                                                          JobDesc="test", 
                                                                          Comments="Now!", 
                                                                          URL="http://www.google.com.au", 
                                                                          CCNumber="4444333322221111", 
                                                                          CCNameOnCard="test", 
                                                                          CCExpiryMonth="07", 
                                                                          CCExpiryYear="12"                                                                          
                                                                          )
        print "update hosted customer", updated_hosted_customer
        self.assertTrue(updated_hosted_customer)
        
    def test_process_payment(self):
        payment_result = self.hosted_test.process_payment("9876543211000", "100", "test", "test")
        print "test_process_payment", payment_result
        self.assertFalse(isinstance(payment_result, WebFault))
        
    def test_query_customer(self):
        query_result = self.hosted_test.query_customer("9876543211000")
        print "test_query_customer", query_result
        self.assertFalse(query_result == None)
        
    def test_query_customer_by_reference(self):
        ref_result = self.hosted_test.query_customer_by_reference("customer reference")
        print "test_query_customer_by_reference", ref_result
        self.assertFalse(ref_result == None)
        
    def test_query_payment(self):
        query_payment_result = self.hosted_test.query_payment("9876543211000") 
        print "test_query_payment", query_payment_result
        self.assertFalse(query_payment_result == None)
        
if __name__ == '__main__':
    unittest.main()


from billing import Integration
from billing.models import GCNewOrderNotification
from django.conf import settings
from xml.dom.minidom import Document
import hmac, hashlib, base64
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.http import HttpResponse
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from django.conf.urls.defaults import patterns
from django.utils.decorators import method_decorator

SANDBOX_URL = 'https://sandbox.google.com/checkout/api/checkout/v2/checkout/Merchant/%s' 
PROD_URL = 'https://checkout.google.com/api/checkout/v2/checkout/Merchant/%s'

BUTTON_SANDBOX_URL = 'http://sandbox.google.com/checkout/buttons/checkout.gif?merchant_id=%(merchant_id)s&w=%(width)s&h=%(height)s&style=white&variant=text&loc=en_US'
BUTTON_URL = 'http://checkout.google.com/checkout/buttons/checkout.gif?merchant_id=%(merchant_id)s&w=%(width)s&h=%(height)s&style=white&variant=text&loc=en_US'

csrf_exempt_m = method_decorator(csrf_exempt)
require_POST_m = method_decorator(require_POST)

class NotConfiguredError(Exception):
    pass

class GoogleCheckoutIntegration(Integration):
    def __init__(self, options=None):
        if not options:
            options = {}
        super(GoogleCheckoutIntegration, self).__init__(options=options)
        if not getattr(settings, "GOOGLE_CHECKOUT_MERCHANT_ID", None):
            raise NotConfiguredError("Could not locate the 'GOOGLE_CHECKOUT_MERCHANT_ID' setting")
        self.merchant_id = settings.GOOGLE_CHECKOUT_MERCHANT_ID
        self._signature = None

    @property
    def service_url(self):
        if self.test_mode:
            return SANDBOX_URL % self.merchant_id
        return PROD_URL % self.merchant_id

    def button_image_url(self):
        params = {"merchant_id": self.merchant_id, 
                  "width": self.button_width,
                  "height": self.button_height}
        if self.test_mode:
            return BUTTON_SANDBOX_URL % params
        return BUTTON_URL % params

    @property
    def button_width(self):
        return self.fields.get("button_width", 180)

    @property
    def button_height(self):
        return self.fields.get("button_height", 46)

    def generate_cart_xml(self):
        doc = Document()
        root = doc.createElement('checkout-shopping-cart')
        root.setAttribute('xmlns', 'http://checkout.google.com/schema/2')
        doc.appendChild(root)
        cart = doc.createElement('shopping-cart')
        root.appendChild(cart)
        items = doc.createElement('items')
        cart.appendChild(items)

        ip_items = self.fields.get("items", [])
        for item in ip_items:
            it = doc.createElement("item")
            items.appendChild(it)
            it_name = doc.createElement("item-name")
            it_name.appendChild(doc.createTextNode(unicode(item["name"])))
            it.appendChild(it_name)
            it_descr = doc.createElement('item-description')
            it_descr.appendChild(doc.createTextNode(unicode(item["description"])))
            it.appendChild(it_descr)
            it_price = doc.createElement("unit-price")
            it_price.setAttribute("currency", unicode(item["currency"]))
            it_price.appendChild(doc.createTextNode(unicode(item["amount"])))
            it.appendChild(it_price)
            it_qty = doc.createElement("quantity")
            it_qty.appendChild(doc.createTextNode(unicode(item["quantity"])))
            it.appendChild(it_qty)
            it_unique_id = doc.createElement("merchant-item-id")
            it_unique_id.appendChild(doc.createTextNode(unicode(item["id"])))
            it.appendChild(it_unique_id)

        checkout_flow = doc.createElement('checkout-flow-support')
        root.appendChild(checkout_flow)
        merchant_checkout_flow = doc.createElement('merchant-checkout-flow-support')
        checkout_flow.appendChild(checkout_flow)
        return_url = doc.createElement('continue-shopping-url')
        return_url.appendChild(doc.createTextNode(self.fields["return_url"]))
        merchant_checkout_flow.appendChild(return_url)

        cart_xml = doc.toxml(encoding="utf-8")
        hmac_signature = hmac.new(settings.GOOGLE_CHECKOUT_MERCHANT_KEY, 
                                  cart_xml, 
                                  hashlib.sha1).digest()
        self._signature = base64.b64encode(hmac_signature)
        return base64.b64encode(cart_xml)

    def signature(self):
        if not self._signature:
            self.generate_cart_xml()
        return self._signature

    @csrf_exempt_m
    @require_POST_m
    def gc_notify_handler(self, request):
        if request.POST['_type'] == 'new-order-notification':
            self.gc_new_order_notification(request)
        elif request.POST['_type'] == 'order-state-change-notification':
            self.gc_order_state_change_notification(request)
        return HttpResponse(request.POST['serial-number'])

    def gc_cart_items_blob(self, post_data):
        items = post_data.getlist('shopping-cart.items')
        cart_blob = ''
        for item in items:
            item_id = post_data.get('%s.merchant-item-id' % (item), '')
            item_name = post_data.get('%s.item-name' % (item), '')
            item_desc = post_data.get('%s.item-description' % (item), '')
            item_price = post_data.get('%s.unit-price' % (item), '')
            item_price_currency = post_data.get('%s.unit-price.currency' % (item), '')
            item_quantity = post_data.get('%s.quantity' % (item), '')
            cart_blob += '%(item_id)s\t%(item_name)s\t%(item_desc)s\t%(item_price)s\t%(item_price_currency)s\t%(item_quantity)s\n\n' % ({"item_id": item_id,
                                                                                                                                         "item_name": item_name,
                                                                                                                                         "item_desc": item_desc,
                                                                                                                                         "item_price": item_price,
                                                                                                                                         "item_price_currency": item_price_currency,
                                                                                                                                         "item_quantity": item_quantity,})
        return cart_blob

    def gc_new_order_notification(self, request):
        post_data = request.POST.copy()
        data = {}

        resp_fields = {
            "_type": "notify_type",
            "serial-number" : "serial_number",      
            "google-order-number" : "google_order_number",
            "buyer-id" : "buyer_id",           
            "buyer-shipping-address.contact-name" : "shipping_contact_name",
            "buyer-shipping-address.address1" : "shipping_address1",    
            "buyer-shipping-address.address2" : "shipping_address2",    
            "buyer-shipping-address.city" : "shipping_city",        
            "buyer-shipping-address.postal-code" : "shipping_postal_code", 
            "buyer-shipping-address.region" : "shipping_region",      
            "buyer-shipping-address.country-code" : "shipping_country_code",
            "buyer-shipping-address.email" : "shipping_email",       
            "buyer-shipping-address.company-name" : "shipping_company_name",
            "buyer-shipping-address.fax" : "shipping_fax",         
            "buyer-shipping-address.phone" : "shipping_phone",       
            "buyer-billing-address.contact-name" : "billing_contact_name",
            "buyer-billing-address.address1" : "billing_address1",    
            "buyer-billing-address.address2" : "billing_address2",    
            "buyer-billing-address.city" : "billing_city",        
            "buyer-billing-address.postal-code" : "billing_postal_code", 
            "buyer-billing-address.region" : "billing_region",      
            "buyer-billing-address.country-code" : "billing_country_code",
            "buyer-billing-address.email" : "billing_email",       
            "buyer-billing-address.company-name" : "billing_company_name",
            "buyer-billing-address.fax" : "billing_fax",         
            "buyer-billing-address.phone" : "billing_phone",       
            "buyer-marketing-preferences.email-allowed" : "marketing_email_allowed",
            "order-adjustment.total-tax" : "total_tax",                
            "order-adjustment.total-tax.currency" : "total_tax_currency",       
            "order-adjustment.adjustment-total" : "adjustment_total",         
            "order-adjustment.adjustment-total.currency" : "adjustment_total_currency",
            "order-total" : "order_total",
            "order-total.currency" : "order_total_currency",
            "financial-order-state" : "financial_order_state",  
            "fulfillment-order-state" : "fulfillment_order_state",
            "timestamp" : "timestamp",
            }
        
        for (key, val) in resp_fields.iteritems():
            data[val] = post_data.get(key, '')

        data['num_cart_items'] = len(post_data.getlist('shopping-cart.items'))
        data['cart_items']     = self.gc_cart_items_blob(post_data)
    
        try:
            resp = GCNewOrderNotification.objects.create(**data)
            # TODO: Make the type more generic
            # TODO: The person might have got charged and yet transaction
            # might have failed here. Need a better way to communicate it
            transaction_was_successful.send(sender=self.__class__, type="purchase", response=resp)
            status = "SUCCESS"
        except:
            transaction_was_unsuccessful.send(sender=self.__class__, type="purchase", response=post_data)
            status = "FAILURE"
        
        return HttpResponse(status)
    

    def gc_order_state_change_notification(self, request):
        post_data = request.POST.copy()
        order = GCNewOrderNotification.objects.get(google_order_number=post_data['google-order-number'])
        order.financial_order_state = post_data['new-financial-order-state']
        order.fulfillment_order_state = post_data['new-fulfillment-order-state']
        order.save()

    def get_urls(self):
        urlpatterns = patterns('',
           (r'^gc-notify-handler/$', self.gc_notify_handler),
                               )
        return urlpatterns



from suds.client import Client, WebFault

REBILL_TEST_URL = "https://www.eway.com.au/gateway/rebill/test/manageRebill_test.asmx?WSDL"
REBILL_LIVE_URL = "https://www.eway.com.au/gateway/rebill/manageRebill.asmx?WSDL"

HOSTED_TEST_URL = "https://www.eway.com.au/gateway/ManagedPaymentService/test/managedCreditCardPayment.asmx?WSDL"
HOSTED_LIVE_URL = "https://www.eway.com.au/gateway/ManagedPaymentService/managedCreditCardPayment.asmx?WSDL"

class RebillEwayClient(object):
    """
    Wrapper for eway payment gateway's managed and rebill webservices
    
    To create a empty object from the webservice types, call self.client.factory.create('type_name')
    
    Useful types are 
        CustomerDetails: rebill customer
        RebillEventDetails: rebill event
        CreditCard: hosted customer
    """
    def __init__(self, test_mode=True, customer_id=None, username=None, password=None, url=None):
        self.gateway_url = REBILL_LIVE_URL
        self.test_mode = test_mode
        self.customer_id = customer_id
        self.username = username
        self.password = password
        if test_mode:
            self.gateway_url = REBILL_TEST_URL
        if url:
            self.gateway_url = url
        self.client = Client(self.gateway_url)
        self.set_eway_header()
        
    def set_eway_header(self):
        """
        creates eway header containing login credentials
        
        required for all api calls
        """
        eway_header = self.client.factory.create("eWAYHeader")
        eway_header.eWAYCustomerID = self.customer_id
        eway_header.Username = self.username
        eway_header.Password = self.password
        self.client.set_options(soapheaders=eway_header)
        
    def create_rebill_customer(self, rebill_customer=None, **kwargs):
        """
        creates rebill customer with CustomerDetails type from the webservice
        
        also accepts keyword arguments if CustomerDetails object is not passed
        return CustomerDetails.RebillCustomerID and CustomerDetails.Result if successfull
        """
        if rebill_customer:
            return self.client.service.CreateRebillCustomer(
                                                             rebill_customer.CustomerTitle,
                                                             rebill_customer.CustomerFirstName,
                                                             rebill_customer.CustomerLastName,
                                                             rebill_customer.CustomerAddress,
                                                             rebill_customer.CustomerSuburb,
                                                             rebill_customer.CustomerState,
                                                             rebill_customer.CustomerCompany,
                                                             rebill_customer.CustomerPostCode,
                                                             rebill_customer.CustomerCountry,
                                                             rebill_customer.CustomerEmail,
                                                             rebill_customer.CustomerFax,
                                                             rebill_customer.CustomerPhone1,
                                                             rebill_customer.CustomerPhone2,
                                                             rebill_customer.CustomerRef,
                                                             rebill_customer.CustomerJobDesc,
                                                             rebill_customer.CustomerComments,
                                                             rebill_customer.CustomerURL,
                                                             )
        else:
            return self.client.service.CreateRebillCustomer(**kwargs)
    
    def update_rebill_customer(self, **kwargs):
        """
        same as create, takes CustomerDetails.RebillCustomerID
        """
        return self.client.service.UpdateRebillCustomer(**kwargs)

    def delete_rebill_customer(self, rebill_customer_id):
        """
        deletes a rebill customer based on id
        """ 
        return self.client.service.DeleteRebillCustomer(rebill_customer_id)
         
    
    def create_rebill_event(self, rebill_event=None, **kwargs):
        """
        creates a rebill event based on RebillEventDetails object
        returns RebillEventDetails.RebillCustomerID and RebillEventDetails.RebillID if successfull
        """
        if rebill_event:
            return self.client.service.CreateRebillEvent(
                                                            rebill_event.RebillCustomerID,
                                                            rebill_event.RebillInvRef,
                                                            rebill_event.RebillInvDesc,
                                                            rebill_event.RebillCCName,
                                                            rebill_event.RebillCCNumber,
                                                            rebill_event.RebillCCExpMonth,
                                                            rebill_event.RebillCCExpYear,
                                                            rebill_event.RebillInitAmt,
                                                            rebill_event.RebillInitDate,
                                                            rebill_event.RebillRecurAmt,
                                                            rebill_event.RebillStartDate,
                                                            rebill_event.RebillInterval,
                                                            rebill_event.RebillIntervalType,
                                                            rebill_event.RebillEndDate,
                                                           )
        else:
            return self.client.service.CreateRebillEvent(**kwargs)

    def update_rebill_event(self, **kwargs):
        """
        same as create, takes RebillEventDetails.RebillCustomerID and RebillEventDetails.RebillID
        """
        return self.client.service.CreateRebillEvent(**kwargs)

    def delete_rebill_event(self, rebill_customer_id, rebill_event_id):
        """
        delete a rebill event based on rebill customer id and event id
        """
        return self.client.service.DeleteRebillEvent(rebill_customer_id, rebill_event_id)
    
    def query_next_transaction(self, RebillCustomerID, RebillID):
        return self.client.service.QueryNextTransaction(RebillCustomerID, RebillID)
    
    def query_rebill_customer(self, RebillCustomerID):
        return self.client.service.QueryRebillCustomer(RebillCustomerID)
    
    def query_rebill_event(self, RebillCustomerID, RebillID):
        return self.client.service.QueryRebillEvent(RebillCustomerID, RebillID)        
    
    def query_transactions(self, RebillCustomerID, RebillID, startDate=None, endDate=None, status=None):
        try:
            return self.client.service.QueryTransactions(RebillCustomerID, RebillID, startDate, endDate, status)
        except WebFault as wf:
            return wf
    
    def create_hosted_customer(self, hosted_customer=None, **kwargs):
        """
        creates hosted customer based on CreditCard type details or kwargs
        
        returns id of newly created customer (112233445566 in test mode) 
        """
        try:
            if hosted_customer:
                return self.client.service.CreateCustomer(
                                                            hosted_customer.Title,
                                                            hosted_customer.FirstName,
                                                            hosted_customer.LastName,
                                                            hosted_customer.Address,
                                                            hosted_customer.Suburb,
                                                            hosted_customer.State,
                                                            hosted_customer.Company,
                                                            hosted_customer.PostCode,
                                                            hosted_customer.Country,
                                                            hosted_customer.Email,
                                                            hosted_customer.Fax,
                                                            hosted_customer.Phone,
                                                            # hosted_customer.Mobile,
                                                            # hosted_customer.CustomerRef,
                                                            # hosted_customer.JobDesc,
                                                            # hosted_customer.Comments,
                                                            # hosted_customer.URL,
                                                            hosted_customer.CCNumber,
                                                            hosted_customer.CCNameOnCard,
                                                            hosted_customer.CCExpiryMonth,
                                                            hosted_customer.CCExpiryYear,
                                                            )
            else:
                return self.client.service.CreateCustomer(**kwargs)
        except WebFault as wf:
            return wf
    
    def update_hosted_customer(self, **kwargs):
        """
        Update hosted customer based on kwargs
        
        returns True or False
        """
        try:
            return self.client.service.UpdateCustomer(**kwargs)
        except WebFault as wf:
            return wf        
    
    def process_payment(self, managedCustomerID, amount, invoiceReference, invoiceDescription):
        """
        makes a transaction based on customer id and amount
        
        returns CCPaymentResponse type object with ewayTrxnStatus, ewayTrxnNumber, ewayAuthCode
        """
        try:
            return self.client.service.ProcessPayment(managedCustomerID, amount, invoiceReference, invoiceDescription)
        except WebFault as wf:
            return wf
    
    def query_customer(self, managedCustomerID):
        return self.client.service.QueryCustomer(managedCustomerID)
    
    def query_customer_by_reference(self, CustomerReference):
        """
        returns customer details based on reference
        
        not working with test data
        """
        return self.client.service.QueryCustomerByReference(CustomerReference)
    
    def query_payment(self, managedCustomerID):
        return self.client.service.QueryPayment(managedCustomerID)




from django import forms
from hashlib import md5
from django.conf import settings

class WPHostedPaymentForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(WPHostedPaymentForm, self).__init__(*args, **kwargs)
        if self.initial:
            self.initial["signatureFields"] = self.initial.get("signatureFields") or "instId:amount:cartId"
            signature_fields = self.initial["signatureFields"].split(":")
            hash_str = ""
            for field in signature_fields:
                hash_str += "%s" % self.initial[field]
                if not signature_fields.index(field) == len(signature_fields) - 1:
                    hash_str += ":"
            md5_hash = md5("%s:%s" %(settings.WORLDPAY_MD5_SECRET_KEY, 
                                     hash_str)).hexdigest()
            self.initial["signature"] = self.initial.get("signature") or md5_hash
    
    # recurring(future pay) parameters
    futurePayType  = forms.CharField(widget=forms.HiddenInput(), required=False)
    intervalUnit   = forms.CharField(widget=forms.HiddenInput(), required=False)
    intervalMult   = forms.CharField(widget=forms.HiddenInput(), required=False)
    option         = forms.CharField(widget=forms.HiddenInput(), required=False)
    noOfPayments   = forms.CharField(widget=forms.HiddenInput(), required=False)
    normalAmount   = forms.CharField(widget=forms.HiddenInput(), required=False)
    startDelayUnit = forms.CharField(widget=forms.HiddenInput(), required=False)
    startDelayMult = forms.CharField(widget=forms.HiddenInput(), required=False)

    instId          = forms.CharField(widget=forms.HiddenInput)
    cartId          = forms.CharField(widget=forms.HiddenInput)
    amount          = forms.DecimalField(widget=forms.HiddenInput)
    currency        = forms.CharField(widget=forms.HiddenInput, initial="USD")
    desc            = forms.CharField(widget=forms.HiddenInput)
    testMode        = forms.CharField(widget=forms.HiddenInput)
    signatureFields = forms.CharField(widget=forms.HiddenInput)
    signature       = forms.CharField(widget=forms.HiddenInput)
    
    #override Country field
    # country = CountryField(initial="AU")



'''
Template tags for Amazon FPS offsite payments
'''
from django import template
from django.template.loader import render_to_string

class AmazonFPSNode(template.Node):
    def __init__(self, integration):
        self.integration = template.Variable(integration)

    def render(self, context):
        int_obj = self.integration.resolve(context)
        form_str = render_to_string("billing/amazon_fps.html", 
                                    {"form": "",
                                     "integration": int_obj}, context)
        return form_str

def amazon_fps(parser, token):
    try:
        tag, int_obj = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r was expecting a single argument" %token.split_contents()[0])
    return AmazonFPSNode(int_obj)



from django.test import TestCase
from billing import get_gateway, CreditCard
from billing.signals import *
from billing.models import EwayResponse
from billing.gateway import CardNotSupported
from billing.utils.credit_card import Visa

fake_options = {
    "email": "testuser@fakedomain.com",
    "billing_address": {
        "name": "PayPal User",
        "address1": "Street 1",
        "city": "Mountain View",
        "state": "CA",
        "country": "US",
        "zip": "94043",
        "fax": "1234567890",
        "email": "testuser@fakedomain.com",
        "phone": "1234567890",
        "mobile": "1234567890",
        "customer_ref": "Blah",
        "job_desc": "Job",
        "comments": "comments",
        "url": "http://google.com.au/",
        },
    "invoice": "1234",
    "description": "Blah Blah!",
}

class EWayGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("eway")
        self.merchant.test_mode = True
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2011, 
                                      number="4444333322221111", 
                                      verification_value="100")

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported, 
                          lambda : self.merchant.purchase(1000, self.credit_card))

    def testCardValidated(self):
        self.merchant.test_mode = False
        self.credit_card.number = "4222222222222123"
        self.assertFalse(self.merchant.validate_card(self.credit_card))

    def testCardType(self):
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card,
                                      options = fake_options)
        # Eway test gateway sets the transaction status as failure
        # in test mode
        self.assertEquals(resp["status"], "FAILURE")
        self.assertEquals(resp["response"].ewayTrxnError, 
                          "1,Do Not Honour(Test Gateway)")
        self.assertNotEquals(resp["response"].ewayTrxnNumber, "0")
        self.assertTrue(resp["response"].ewayReturnAmount, "1") 

    def testPaymentSuccessfulSignal(self):
        # Since in the test mode, all transactions are
        # failures, we need to be checking for transaction_was_unsuccessful
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_unsuccessful.connect(receive)

        resp = self.merchant.purchase(1, self.credit_card,
                                      options = fake_options)
        self.assertEquals(received_signals, [transaction_was_unsuccessful])

    def testPaymentUnSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_unsuccessful.connect(receive)

        resp = self.merchant.purchase(6, self.credit_card,
                                      options = fake_options)
        self.assertEquals(received_signals, [transaction_was_unsuccessful])

    def testCreditCardExpired(self):
        resp = self.merchant.purchase(8, self.credit_card,
                                      options = fake_options)
        self.assertNotEquals(resp["status"], "SUCCESS")



from authorize_models import AuthorizeAIMResponse
from gc_models import GCNewOrderNotification
from world_pay_models import WorldPayResponse
from eway_models import EwayResponse
from amazon_fps_models import AmazonFPSResponse
